@@ -576,7 +576,11 @@ user aborted."
                                                             (backtrace-string) ""))))))
               (parse-and-run-command cmd))
           (eval-command-error (err-text)
-            :interactive (lambda () nil)
+            :interactive (lambda ()
+                           (list (format nil "^B^1*Error In Command '^b~a^B'"
+                                         cmd)))
+            :report (lambda (s)
+                      (format s "Exit command ~S" cmd))
             (values err-text t)))
       ;; interactive commands update the modeline
       (update-all-mode-lines)
