@@ -82,10 +82,16 @@ function expects to be wrapped in a with-state for win."
                  (* (xlib:drawable-border-width win) 2)))
            (head-x (head-x (current-head)))
            (head-y (head-y (current-head)))
-           (head-maxx (+ head-x (head-width (current-head))))
-           (head-maxy (+ head-y (head-height (current-head)))))
+           (head-width (head-width (current-head)))
+           (head-height (head-height (current-head)))
+           (margin-x (if (> (* *message-window-margin* 2) head-width) 0 *message-window-margin*))
+           (margin-y (if (> (* *message-window-y-margin* 2) head-height) 0 *message-window-y-margin*))
+           (minx (+ head-x margin-x))
+           (miny (+ head-y margin-y))
+           (maxx (- (+ head-x head-width) margin-x))
+           (maxy (- (+ head-y head-height) margin-y)))
       (multiple-value-bind (x y)
-          (gravity-coords gravity w h head-x head-y head-maxx head-maxy)
+          (gravity-coords gravity w h minx miny maxx maxy)
         (setf (xlib:drawable-y win) (max head-y y)
               (xlib:drawable-x win) (max head-x x))))))
 
@@ -277,6 +283,7 @@ When NEW-ON-BOTTOM-P is non-nil, new messages are queued at the bottom."
 (defun echo-string-list (screen strings &rest highlights)
   "Draw each string in l in the screen's message window. HIGHLIGHT is
   the nth entry to highlight."
+  (setf highlights (delete nil highlights))
   (when strings
     (when *queue-messages-p*
       (multiple-value-bind (combined-strings combined-highlights)
