@@ -363,14 +363,14 @@ modes are enabled in them, then nullify the list of objects."
 (defun replace-class-and-sync (object new-class &rest initargs)
   "Replaces the main class in OBJECT with the new class, and then syncs all minor
 modes."
-  (apply #'dynamic-mixins:replace-class object new-class initargs)
+  (apply #'dynamic-mixins-swm:replace-class object new-class initargs)
   (sync-minor-modes object))
 
 (defun list-modes (object)
   "List all minor modes followed by the major mode for OBJECT."
   (sync-all-minor-modes)
-  (when (typep object 'dynamic-mixins:mixin-object)
-    (mapcar #'class-name (dynamic-mixins:mixin-classes (class-of object)))))
+  (when (typep object 'dynamic-mixins-swm:mixin-object)
+    (mapcar #'class-name (dynamic-mixins-swm:mixin-classes (class-of object)))))
 
 (defun list-minor-modes (object)
   "List all minor modes active in OBJECT"
@@ -601,7 +601,9 @@ ROOT-MAP-SPEC."
               (:enable-when            . t)
               (:make-hooks             . 1)
               (:default-initargs       . t)
-              (:define-command-definer . 1)))
+              (:define-command-definer . 1)
+              (:mix-before             . t)
+              (:mix-after              . t)))
           (all-vals '())
           (other-opts '()))
       (flet ((collect-values (option)
@@ -642,7 +644,7 @@ ROOT-MAP-SPEC."
                          ;; conforms to that type.
                          `((typep obj ',(third optarg))))
                      (enable-when mode obj))
-            (prog1 (dynamic-mixins:ensure-mix obj ',mode)
+            (prog1 (dynamic-mixins-swm:ensure-mix obj ',mode)
               (handler-bind ((minor-mode-hook-error
                                (lambda (c)
                                  (let ((r (find-restart 'continue c)))
@@ -658,7 +660,7 @@ ROOT-MAP-SPEC."
                                (when r
                                  (invoke-restart r))))))
             (run-hook-for-minor-mode #'minor-mode-disable-hook ',mode obj t))
-          (dynamic-mixins:delete-from-mix obj ',mode)))))
+          (dynamic-mixins-swm:delete-from-mix obj ',mode)))))
 
   (defun genlighter (mode lighter)
     (cond ((null lighter)
@@ -761,10 +763,13 @@ DESIGNATOR in the minor mode scope hash table."
   (defun scope-current-object-function (designator)
     (cadr (get-scope designator)))
   (defun scope-all-objects-function (designator)
-    (let ((type (first (get-scope designator))))
+    (let* ((scope (get-scope designator))
+           (type (first scope))
+           (filter (third scope)))
       (lambda ()
         (loop for object in (list-mode-objects nil)
-              when (typep object type)
+              when (and (typep object type)
+                        (typep object filter))
                 collect object))))
   (defun find-active-global-minor-modes-for-scope (scope)
     (loop for mode in *active-global-minor-modes*
@@ -878,12 +883,9 @@ provided."
 (define-minor-mode-scope (:dynamic-group dynamic-group)
   (current-group))
 
-(defun %manual-tiling-group-p (g)
-  (and (typep g 'tile-group)
-       (not (typep g 'dynamic-group))))
-
-(define-minor-mode-scope (:manual-tiling-group tile-group
-                                               (satisfies %manual-tiling-group-p))
+(define-minor-mode-scope (:manual-tiling-group
+                          tile-group
+                          (and tile-group (not dynamic-group)))
   (current-group))
 
 (define-minor-mode-scope (:frame frame)
@@ -894,12 +896,8 @@ provided."
 (define-minor-mode-scope (:head head)
   (current-head))
 
-(defun %frame-but-not-head (o)
-  (and (typep o 'frame)
-       (not (typep o 'head))))
-
 (define-descended-minor-mode-scope :frame-excluding-head :frame
-  :filter-type (satisfies %frame-but-not-head))
+  :filter-type (and frame (not head)))
 
 (define-minor-mode-scope (:window window)
   (current-window))
@@ -1017,6 +1015,24 @@ are active only when the minor mode is active. Commands defined with this macro
 have the special variable *MINOR-MODE* bound to the minor mode object in their
 body. The generated macro is called DEFINE-MODE-COMMAND. This option defaults to
 T.
+
+@item
+(:MIX-BEFORE &REST RULES)@*
+The :MIX-BEFORE option defines rules on the order this class should be mixed in
+relative to other minor modes. This allows the implementer of a minor mode to
+make the mixing process aware of dependencies that dont otherwise make sense as
+a class hierarchy; If minor modes FOO and BAR both define around methods for the
+same method, but FOO's method must be called first, FOO can add a rule stating
+that it must come before BAR in the mixin list. RULES must be a set of conses
+which have the form (SYMBOL-DESIGNATOR . PACKAGE-DESIGNATOR). SYMBOL-DESIGNATOR
+must be a valid argument to #'STRING, and PACKAGE-DESIGNATOR must be a valid
+argument to #'FIND-PACKAGE. Together these shall form a single symbol which
+should be the class name of the minor mode being referred to by the rule.
+
+@item
+(:MIX-AFTER &REST RULES)@*
+The :MIX-AFTER option is similar to the :MIX-BEFORE option, except it specifies
+classes that this minor mode should occur after in the mixin list.
 @end itemize
 
 Example:
@@ -1043,7 +1059,7 @@ Example:
     (destructuring-bind (&key top-map root-map (expose-keymaps t) rebind
                            lighter lighter-make-clickable lighter-on-click
                            (scope :unscoped) interactive global
-                           (enable-when nil ewpp) 
+                           (enable-when nil ewpp) mix-before mix-after
                            (make-hooks t) (define-command-definer t)
                            default-initargs)
         mm-opts
@@ -1087,6 +1103,18 @@ Example:
               ,@slots)
              (:default-initargs ,@default-initargs)
              ,@other-opts)
+
+           ,@(when (or mix-after mix-before)
+               (flet ((mkc (s)
+                        (list 'cons (car s) (cdr s))))
+                 (let ((mix-a (mapcar #'mkc mix-after))
+                       (mix-b (mapcar #'mkc mix-before)))
+                   ;; Convert to explicit #'CONS calls to allow destructive
+                   ;; modification of data at runtime.
+                   `((dynamic-mixins-swm::set-rule ',mode
+                                                   (list ,@mix-b)
+                                                   (list ,@mix-a))))))
+           
            ,(if global 
                 `(defmethod minor-mode-global-p ((mode (eql ',mode))) t)
                 `(let ((method (ignore-errors
