@@ -351,6 +351,19 @@ timer.")
   "Update all mode lines."
   (mapc 'redraw-mode-line *mode-lines*))
 
+(defun refresh-colors-for-modeline (screen head)
+  "Set the colors for the modeline to the currently set values then redraw"
+  (let* ((ml (head-mode-line head))
+         (cc (mode-line-cc ml))
+         (win (mode-line-window ml)))
+    (setf (ccontext-default-bg cc)
+          (alloc-color screen *mode-line-background-color*))
+    (setf (ccontext-default-fg cc)
+          (alloc-color screen *mode-line-foreground-color*))
+    (setf (xlib:window-background win)
+          (alloc-color screen *mode-line-background-color*))
+    (redraw-mode-line ml t)))
+
 ;;; Registering mode line clickable areas
 
 (defvar *mode-line-on-click-functions* nil
