@@ -649,6 +649,31 @@ and bottom_end_x."
   "Return a free window number for GROUP."
   (find-free-number (mapcar 'window-number (group-windows group))))
 
+(defun draw-window-numbers (group)
+  "Draw window numbers over every window at the coordinates (0 . 0)."
+  (let ((screen (group-screen group)))
+    (mapcar (lambda (f)
+              (let ((w (xlib:create-window
+                        :parent (screen-root screen)
+                        :x (window-x f)
+                        :y (window-y  f)
+                        :width 1
+                        :height 1
+                        :background (screen-fg-color screen)
+                        :border (screen-border-color screen)
+                        :border-width 1
+                        :event-mask '())))
+                (xlib:map-window w)
+                (setf (xlib:window-priority w) :above)
+                (echo-in-window w (screen-font screen)
+                                (screen-fg-color screen)
+                                (screen-bg-color screen)
+                                (string (format nil "~A" (window-number f))))
+                (xlib:display-finish-output *display*)
+                (dformat 3 "mapped ~S~%" (window-number f))
+                w))
+            (group-windows group))))
+
 (defun reparent-window (screen window)
   ;; apparently we need to grab the server so the client doesn't get
   ;; the mapnotify event before the reparent event. that's what fvwm
@@ -780,12 +805,11 @@ and bottom_end_x."
   "Any time *top-map* is modified this must be called."
   (loop for i in *screen-list*
         do (xwin-ungrab-keys (screen-focus-window i))
-        do (loop for j in (screen-mapped-windows i)
+           (loop for j in (screen-mapped-windows i)
                  do (xwin-ungrab-keys j))
-        do (xlib:display-finish-output *display*)
-        do (loop for j in (screen-mapped-windows i)
+           (loop for j in (screen-mapped-windows i)
                  do (xwin-grab-keys j (window-group (find-window j))))
-        do (xwin-grab-keys (screen-focus-window i) (screen-current-group i)))
+           (xwin-grab-keys (screen-focus-window i) (screen-current-group i)))
   (when (current-window)
     (remap-keys-grab-keys (current-window)))
   (xlib:display-finish-output *display*))
@@ -1229,14 +1253,18 @@ formatting. This is a simple wrapper around the command @command{windowlist}."
 
 (defcommand-alias insert window-send-string)
 
-(defcommand mark () ()
-"Toggle the current window's mark."
-  (let ((win (current-window)))
-    (when win
-      (setf (window-marked win) (not (window-marked win)))
+(defcommand mark (&optional (win (current-window)) (message t)) ()
+"Toggle a window's mark. The optional argument WIN controls which window is
+marked and defaults to the current window. The optional argument MESSAGE
+controls whether or not to display a message to the user indicating that WIN has
+been marked, and defaults to T."
+  (when win
+    (setf (window-marked win) (not (window-marked win)))
+    (when message
       (message (if (window-marked win)
-                   "Marked!"
-                   "Unmarked!")))))
+                   "^3~A^n Marked!"
+                   "^3~A^n Unmarked!")
+               (format-expand *window-formatters* *window-format* win)))))
 
 (defcommand clear-window-marks (&optional (group (current-group)) (windows (group-windows group))) ()
 "Clear all marks in the current group."
